(* this file contains all inits of variables used in the demo *)

(* appwindow.inc *)
  appwindow_registered := FALSE;
  application_window   := NULL;


(* button_box.inc *)
  bbox_window := NULL;

(* colorsel.inc *)
  colorsel_window := NULL;
  colorsel_da     := NULL;
  colorsel_frame  := NULL;

(* drawingarea.inc *)
  da_window  := NULL;

(* images.inc *)
  image_window        := NULL;
  image_pixbuf_loader := NULL;
  image_load_timeout  := 0;

(* item_factory *)
  if_window  := NULL;

(* menus.inc *)
  menu_window := NULL;

(* panes.inc *)
  panes_window := NULL;

(* pixbufs.inc *)
  pixbufs_window      := NULL;
  pixbufs_frame       := NULL;
  pixbufs_background  := NULL;
  pixbufs_back_width  := 0;
  pixbufs_back_height := 0;
  pixbufs_da          := NULL;

(* sizegroup.inc *)
  sg_window           := NULL;

(* stock_browser.inc *)
  stock_window        := NULL;

(* textview.inc *)
  text_window         := NULL;
  text_egg_window     := NULL;



